/// <reference types="@applicaster/applicaster-types" />
import React from "react";
import * as R from "ramda";
import { View } from "react-native";
import { usePickFromState } from "@applicaster/zapp-react-native-redux/hooks";
import {
  isOrientationCompatible,
  useGetScreenOrientation,
} from "@applicaster/zapp-react-native-utils/appUtils/orientationHelper";

import { useTheme } from "@applicaster/zapp-react-native-utils/theme";
import { getComponentModule } from "@applicaster/zapp-react-native-utils/pluginUtils";
import {
  getNavigationPropsV2,
  resolveNavigationPlugin,
} from "@applicaster/zapp-react-native-utils/navigationUtils";
import { useSafeAreaInsets } from "react-native-safe-area-context";
import {
  useDimensions,
  useRoute,
  useCurrentScreenData,
  useNavbarState,
  useScreenData,
  useNavigation,
} from "@applicaster/zapp-react-native-utils/reactHooks";
import { getNavigationPluginModule } from "@applicaster/zapp-react-native-app/App/Layout/layoutHelpers";

import { ScreenContextProvider } from "../../Contexts/ScreenContext";
import { RouteManager } from "../RouteManager";
import { useScreenConfiguration } from "../River/useScreenConfiguration";
import { useScreenOrientationHandler } from "./orientationHandler";

const getScreenId = (
  screenData: LegacyNavigationScreenData,
  fallbackRiver: ZappRiver
) => {
  return screenData?.targetScreen
    ? screenData?.targetScreen?.id ?? fallbackRiver.id
    : "screen_id" in screenData
    ? screenData.screen_id
    : fallbackRiver.id;
};

function colorIsNotTransparent(color) {
  const layers = R.compose(
    R.map(R.trim),
    R.split(","),
    R.replace(")", ""),
    R.replace("rgba(", "")
  )(color);

  const alpha = Number(layers?.[3]);

  return alpha > 0;
}

function isValidColor(string) {
  return string && string !== "transparent" && colorIsNotTransparent(string);
}

type Props = {
  pathname: unknown;
  screenData: unknown;
};

export function Screen(_props: Props) {
  const theme = useTheme<BaseThemePropertiesMobile>();
  const navigation = useNavigation();
  const { plugins } = usePickFromState(["plugins"]);

  // Gets the data for the current screen configuration
  const currentScreenData = useCurrentScreenData();
  const { backgroundColor } = useScreenConfiguration(currentScreenData.id);
  const insets = useSafeAreaInsets();

  const {
    width: screenWidth,
    height,
    deviceInfo,
  } = useDimensions("screen", {
    fullDimensions: true,
  });

  const { screenData, pathname } = useRoute();

  const currentRiver = useScreenData(
    getScreenId(screenData, navigation.activeRiver)
  );

  const { title } = useNavbarState();

  const isTablet = deviceInfo?.isTablet;

  const { appData } = usePickFromState(["appData"]);
  const isTabletPortrait = appData?.isTabletPortrait;

  const layoutData = { isTablet, isTabletPortrait, width: screenWidth, height };

  const hasMenu =
    resolveNavigationPlugin({
      category: "menu",
      navigations: currentRiver.navigations,
      plugins,
    })?.identifier === "quick_brick_side_menu";

  const navBarProps = React.useMemo<MobileNavBarPluginProps | null>(() => {
    const props = getNavigationPropsV2({
      currentRiver,
      title,
      category: "nav_bar",
    });

    if (props) {
      return {
        ...props,
        id: pathname,
        pathname: pathname,
      };
    }

    return null;
  }, [currentRiver, pathname]);

  const NavBar = React.useMemo(
    () =>
      getNavigationPluginModule(
        "nav_bar",
        currentRiver,
        plugins
      ) as React.ComponentType<MobileNavBarPluginProps & { hasMenu: boolean }>,
    []
  );

  const offlinePlugin = getComponentModule({
    plugins,
    components: {},
    componentType: "offline-experience",
  });

  const { OfflineFallbackScreen } = offlinePlugin;

  const style = React.useMemo(
    () => ({
      flex: 1,
      backgroundColor: isValidColor(backgroundColor)
        ? backgroundColor
        : theme.app_background_color,
      paddingTop: 0,
      // paddingBottom: insets.bottom, // Should be applied directly by UI plugin
      // paddingLeft: insets.left, // Should be applied directly by UI plugin
      // paddingRight: insets.right, // Should be applied directly by UI plugin
    }),
    [insets, screenWidth, theme.app_background_color, backgroundColor]
  );

  const targetScreenData =
    currentScreenData || (screenData as any)?.targetScreen || screenData;

  useScreenOrientationHandler({ screenData: targetScreenData });

  const orientation = useGetScreenOrientation(targetScreenData);

  // We prevent rendering of the screen until UI is actually rotated to screen desired orientation.
  // This saves unnecessary re-renders and user will not see distorted aspect screen.
  if (
    !isOrientationCompatible({
      orientation,
      layoutData,
    })
  ) {
    return <View style={style} />;
  }

  return (
    <ScreenContextProvider pathname={pathname}>
      <View style={style}>
        {navBarProps && <NavBar {...navBarProps} hasMenu={hasMenu} />}

        <OfflineFallbackScreen screenData={targetScreenData}>
          {/* @TODO RouteManager doesn't use props, can they be removed ?  */}
          <RouteManager pathname={pathname} screenData={screenData} />
        </OfflineFallbackScreen>
      </View>
    </ScreenContextProvider>
  );
}
